%  hStrain.m
%  Statistical strain analysis:
%  Bootstrap method for estimating strain and its error from R/phi data
%  on the unit hyperboloid

clear all;
d2r = pi / 180; r2d = 1 / d2r;
clf;

%%%%%%%%%%%%
Alpha = 0.95 ;   % confidence level
%%%%%%%%%%%%

[filename,pathname] = uigetfile('*.txt', 'a_f b_f phi_f');
Rfphi_dat = load(fullfile(pathname,filename));
filename

n    = length( Rfphi_dat );     % number of data
rhof = zeros(n,1);
psif = zeros(n,1);
xf   = zeros(n,3);

%%% determination of optimal strain
[rhof, psif] = Rphi2rhopsi( Rfphi_dat(:,1) ./ Rfphi_dat(:,2), ...
                            Rfphi_dat(:,3)*d2r );
xf = rhopsi2vec( rhof, psif );
resultvector = sum(xf);
resultlength = sqrt(-HyperbolicInnerProd(resultvector, ...
                                         resultvector));
xs = resultvector / resultlength;
  [rhos, psis] = vec2rhopsi(xs);
  [Rs, phis] = rhopsi2Rphi(rhos,psis);

%%%% Bootstrap solutions
nb = 10 * n; % number of bootstrap samples
nc = floor( Alpha * nb ); % number of solutions within region
xb   = zeros(n,3);
xsb  = zeros(nb,5); % 4th column contains distances from xs on 12-plane
for ib = 1 : nb
    r = floor( rand(n,1) * n ) + 1;
    xb = xf(r,:);
    resultvector = sum(xb);
    resultlength = sqrt(-HyperbolicInnerProd(resultvector, ...
                                             resultvector));
    xsb(ib,1:3) = resultvector / resultlength;
    xsb(ib,4)   = HyperbolicDistance( xs, xsb(ib,1:3) );
end
xsb = sortrows( xsb, 4 );
px = EqualAreaProjection_from_H2vec( xsb );
plot( px(:,1), px(:,2), 'c.' );


%%%% Determination of confidence region
xsb = xsb( 1:nc, : );  % bootstrap solutions within confidence region
[rho, psi] = vec2rhopsi(xsb(:,1:3));
rhomx = max(rho); psimx = max(psi);
rhomn = min(rho); psimn = min(psi);
[Rmx,phimx] = rhopsi2Rphi( rhomx, psimx );
[Rmn,phimn] = rhopsi2Rphi( rhomn, psimn );
D1R = Rmx - Rs; D1phi = phimx - phis;
D2R = Rs - Rmn; D2phi = phis - phimn;
fprintf(1, 'Number of data = %5d\n', n);
fprintf(1, 'R_s   = %5.3f +%5.3f/-%5.3f\n', Rs, D1R, D2R)
fprintf(1, 'Phi_s = %5.3f +%5.3f/-%5.3f\n', phis*r2d, D1phi*r2d, D2phi*r2d)

direc = xsb(:,2:3) - [xs(2)*ones(nc,1), xs(3)*ones(nc,1)]; % rays from xs, optimal solution
xsb(:,5) = atan2( direc(:,2), direc(:,1) );% + 2*pi;         % pi < xsb(:,5) < 3*pi
xsb = sortrows( xsb, 5 );                                  % sorting by direction from xs

envelope = zeros(13,3);
[c, j] = max( xsb(1:floor(nc/12),4) );
envelope(1, :) = xsb(j,1:3);
envelope(13,:) = xsb(j,1:3);
for i = 2 : 11
    j0 = floor(nc/12.0*(i-1));
    j1 = floor(nc/12.0*i    );
    [c, j] = max( xsb(j0:j1,4) );
    envelope(i,:) = xsb(j+j0-1,1:3);
end
j0 = floor(nc/12.0*11);
[c, j] = max( xsb(j0:nc, 4 ) );
envelope(12,:) = xsb(j+j0-1,1:3);

hold on

% Rf/phi data
pf = EqualAreaProjection_from_H2vec( xf );
xlm = max(xf(:,1));
plot(pf(:,1), pf(:,2), 'mo', 'MarkerSize', 5);
% bootstrap solutions
px = EqualAreaProjection_from_H2vec( xsb );
% confidence region
pb = EqualAreaProjection_from_H2vec( envelope );
plot( pb(:,1), pb(:,2), 'b-' );
% optomal solution
ps = EqualAreaProjection_from_H2vec( xs );
plot( ps(1), ps(2), 'r+' );

axis equal